#include "Fbo.h"
#include <iostream>

using namespace cgue::thewall;

Fbo::Fbo(int _WIDTH, int _HEIGHT){
	WIDTH = _WIDTH;
	HEIGHT = _HEIGHT;

	g_quad_vertex_buffer_data[0] = -1.0f;
	g_quad_vertex_buffer_data[1] = -1.0f;
	g_quad_vertex_buffer_data[2] = 0.0f;
	g_quad_vertex_buffer_data[3] = 1.0f;
	g_quad_vertex_buffer_data[4] = -1.0f;
	g_quad_vertex_buffer_data[5] = 0.0f;
	g_quad_vertex_buffer_data[6] = -1.0f;
	g_quad_vertex_buffer_data[7] = 1.0f;
	g_quad_vertex_buffer_data[8] = 0.0f;
	g_quad_vertex_buffer_data[9] = -1.0f;
	g_quad_vertex_buffer_data[10] = 1.0f;
	g_quad_vertex_buffer_data[11] = 0.0f;
	g_quad_vertex_buffer_data[12] = 1.0f;
	g_quad_vertex_buffer_data[13] = -1.0f;
	g_quad_vertex_buffer_data[14] = 0.0f;
	g_quad_vertex_buffer_data[15] = 1.0f;
	g_quad_vertex_buffer_data[16] = 1.0f;
	g_quad_vertex_buffer_data[17] = 0.0f;
	
	initRenderedSceneTexture();
	initAlphaTexture();

	//Hier wird der Buffer fr den Quad initialisiert
	glGenBuffers(1, &quad_vertexbuffer);
	glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(g_quad_vertex_buffer_data), g_quad_vertex_buffer_data, GL_STATIC_DRAW);
	
}

//Dieser Buffer erzeugt zwei Texturen, einmal die Szene, und einmal nur die teile der Szene, deren Alpha Kanal 0 ist.
void Fbo::initRenderedSceneTexture(){

	glGenTextures(1, &textureId);
	glBindTexture(GL_TEXTURE_2D, textureId),
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE); // automatic mipmap
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, WIDTH, HEIGHT, 0,
				 GL_RGBA, GL_UNSIGNED_BYTE, 0);
	glBindTexture(GL_TEXTURE_2D, 0);

	glGenTextures(1, &alphaTexturId);
	glBindTexture(GL_TEXTURE_2D, alphaTexturId),
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE); // automatic mipmap
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, WIDTH, HEIGHT, 0,
				 GL_RGBA, GL_UNSIGNED_BYTE, 0);
	glBindTexture(GL_TEXTURE_2D, 0);


	// create a renderbuffer object to store depth info
	GLuint rboId;
	glGenRenderbuffers(1, &rboId);
	glBindRenderbuffer(GL_RENDERBUFFER, rboId);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, WIDTH, HEIGHT);
	glBindRenderbuffer(GL_RENDERBUFFER, 0);

	// create a framebuffer object

	glGenFramebuffers(1, &fboId);
	glBindFramebuffer(GL_FRAMEBUFFER, fboId);

	// attach the texture to FBO color attachment point
	glFramebufferTexture2D(GL_FRAMEBUFFER,        // 1. fbo target: GL_FRAMEBUFFER 
						   GL_COLOR_ATTACHMENT0,  // 2. attachment point
						   GL_TEXTURE_2D,         // 3. tex target: GL_TEXTURE_2D
						   textureId,             // 4. tex ID
						   0);                    // 5. mipmap level: 0(base)

	glFramebufferTexture2D(GL_FRAMEBUFFER,        // 1. fbo target: GL_FRAMEBUFFER 
						   GL_COLOR_ATTACHMENT1,  // 2. attachment point
						   GL_TEXTURE_2D,         // 3. tex target: GL_TEXTURE_2D
						   alphaTexturId,             // 4. tex ID
						   0);                    // 5. mipmap level: 0(base)

	 GLenum buffers[] = {GL_COLOR_ATTACHMENT0, GL_COLOR_ATTACHMENT1};

	// attach the renderbuffer to depth attachment point
	glFramebufferRenderbuffer(GL_FRAMEBUFFER,      // 1. fbo target: GL_FRAMEBUFFER
							  GL_DEPTH_ATTACHMENT, // 2. attachment point
							  GL_RENDERBUFFER,     // 3. rbo target: GL_RENDERBUFFER
							  rboId);              // 4. rbo ID

	// check FBO status
	GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER);
	if(status != GL_FRAMEBUFFER_COMPLETE){
		std::cout<<"Beim erstellen des Framebuffers ist ein Fehler aufgetreten"<<std::endl;
		system("PAUSE");
		exit(-1);
	}

	glBindFramebuffer(GL_FRAMEBUFFER,0);

}

void Fbo::initAlphaTexture(){

	//Diese Textur speichert den ersten Gauss Durchgang
	glGenTextures(1, &gaussHorizontalTexturId);
	glBindTexture(GL_TEXTURE_2D, gaussHorizontalTexturId),
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE); // automatic mipmap
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, WIDTH, HEIGHT, 0,
				 GL_RGBA, GL_UNSIGNED_BYTE, 0);
	glBindTexture(GL_TEXTURE_2D, 0);

	glGenTextures(1, &gaussFinalTexturId);
	glBindTexture(GL_TEXTURE_2D, gaussFinalTexturId),
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE); // automatic mipmap
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, WIDTH, HEIGHT, 0,
				 GL_RGBA, GL_UNSIGNED_BYTE, 0);
	glBindTexture(GL_TEXTURE_2D, 0);


	// create a renderbuffer object to store depth info
	GLuint rboId;
	glGenRenderbuffers(1, &rboId);
	glBindRenderbuffer(GL_RENDERBUFFER, rboId);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, WIDTH, HEIGHT);
	glBindRenderbuffer(GL_RENDERBUFFER, 0);
	
	// create a framebuffer object

	glGenFramebuffers(1, &fboGaussId);
	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);

	// attach the texture to FBO color attachment point
	glFramebufferTexture2D(GL_FRAMEBUFFER,        // 1. fbo target: GL_FRAMEBUFFER 
						   GL_COLOR_ATTACHMENT0,  // 2. attachment point
						   GL_TEXTURE_2D,         // 3. tex target: GL_TEXTURE_2D
						   gaussHorizontalTexturId,             // 4. tex ID
						   0);                    // 5. mipmap level: 0(base)

	glFramebufferTexture2D(GL_FRAMEBUFFER,        // 1. fbo target: GL_FRAMEBUFFER 
						   GL_COLOR_ATTACHMENT1,  // 2. attachment point
						   GL_TEXTURE_2D,         // 3. tex target: GL_TEXTURE_2D
						   gaussFinalTexturId,             // 4. tex ID
						   0);                    // 5. mipmap level: 0(base)

	 GLenum buffers[] = {GL_COLOR_ATTACHMENT0, GL_COLOR_ATTACHMENT1};

	// attach the renderbuffer to depth attachment point
	glFramebufferRenderbuffer(GL_FRAMEBUFFER,      // 1. fbo target: GL_FRAMEBUFFER
							  GL_DEPTH_ATTACHMENT, // 2. attachment point
							  GL_RENDERBUFFER,     // 3. rbo target: GL_RENDERBUFFER
							  rboId);              // 4. rbo ID

	// check FBO status
	GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER);
	if(status != GL_FRAMEBUFFER_COMPLETE){
		std::cout<<"Beim erstellen des Framebuffers ist ein Fehler aufgetreten"<<std::endl;
		system("PAUSE");
		exit(-1);
	}

	glBindFramebuffer(GL_FRAMEBUFFER,0);

}

void Fbo::drawGaussTex(Shader* gaussHorizontalShader, Shader* gaussFinalShader){

	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);

	glDrawBuffer(GL_COLOR_ATTACHMENT0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glBindFramebuffer(GL_FRAMEBUFFER,0);

	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);
	glDrawBuffer(GL_COLOR_ATTACHMENT1);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glBindFramebuffer(GL_FRAMEBUFFER,0);

	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);
	
	//Zuerst der horizontale Gauss filter
	glDrawBuffer(GL_COLOR_ATTACHMENT0);

	gaussHorizontalShader->useShader();
	
	//Hier gebe ich die Alphatextur in die color Map
	GLuint aplhatexID = glGetUniformLocation(gaussHorizontalShader->programHandle, "colorMap");
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, alphaTexturId);
	glUniform1i(aplhatexID , 0);
	
	// 1rst attribute buffer : vertices
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
		glVertexAttribPointer(
			0,                  // attribute 0. No particular reason for 0, but must match the layout in the shader.
			3,                  // size
			GL_FLOAT,           // type
			GL_FALSE,           // normalized?
			0,                  // stride
			(void*)0            // array buffer offset
		);

		// Draw the triangles !
	glDrawArrays(GL_TRIANGLES, 0, 6); // 2*3 indices starting at 0 -> 2 triangles

	glDisableVertexAttribArray(0);
	
	//Jetzt wird noch der vertikale Gauss Filter angewandt-----------------------------------------------------------
	
	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);
	
	//Zuerst der horizontale Gauss filter
	glDrawBuffer(GL_COLOR_ATTACHMENT1);

	gaussFinalShader->useShader();
	
	//Hier gebe ich die Alphatextur in die color Map
	GLuint FinalaplhatexID = glGetUniformLocation(gaussFinalShader->programHandle, "colorMap");
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, gaussHorizontalTexturId);
	glUniform1i(FinalaplhatexID , 0);

	// 1rst attribute buffer : vertices
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
		glVertexAttribPointer(
			0,                  // attribute 0. No particular reason for 0, but must match the layout in the shader.
			3,                  // size
			GL_FLOAT,           // type
			GL_FALSE,           // normalized?
			0,                  // stride
			(void*)0            // array buffer offset
		);

		// Draw the triangles !
	glDrawArrays(GL_TRIANGLES, 0, 6); // 2*3 indices starting at 0 -> 2 triangles

	glDisableVertexAttribArray(0);

	//Zweiter Durchlauf--------------------------------------------------------------------------------------------------------

	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);

	//glDrawBuffer(GL_COLOR_ATTACHMENT0);
	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	//glBindFramebuffer(GL_FRAMEBUFFER,0);

	//glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	//glViewport(0,0,WIDTH, HEIGHT);
	//glDrawBuffer(GL_COLOR_ATTACHMENT1);
	//glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	//glBindFramebuffer(GL_FRAMEBUFFER,0);

	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);
	
	//Zuerst der horizontale Gauss filter
	glDrawBuffer(GL_COLOR_ATTACHMENT0);

	gaussHorizontalShader->useShader();
	
	//Hier gebe ich die Alphatextur in die color Map
	GLuint aplhatexID2 = glGetUniformLocation(gaussHorizontalShader->programHandle, "colorMap");
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, gaussFinalTexturId);
	glUniform1i(aplhatexID2 , 0);
	
	// 1rst attribute buffer : vertices
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
		glVertexAttribPointer(
			0,                  // attribute 0. No particular reason for 0, but must match the layout in the shader.
			3,                  // size
			GL_FLOAT,           // type
			GL_FALSE,           // normalized?
			0,                  // stride
			(void*)0            // array buffer offset
		);

		// Draw the triangles !
	glDrawArrays(GL_TRIANGLES, 0, 6); // 2*3 indices starting at 0 -> 2 triangles

	glDisableVertexAttribArray(0);
	
	//Jetzt wird noch der vertikale Gauss Filter angewandt-----------------------------------------------------------
	
	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);
	
	//Zuerst der horizontale Gauss filter
	glDrawBuffer(GL_COLOR_ATTACHMENT1);

	gaussFinalShader->useShader();
	
	//Hier gebe ich die Alphatextur in die color Map
	GLuint FinalaplhatexID2 = glGetUniformLocation(gaussFinalShader->programHandle, "colorMap");
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, gaussHorizontalTexturId);
	glUniform1i(FinalaplhatexID2 , 0);

	// 1rst attribute buffer : vertices
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
		glVertexAttribPointer(
			0,                  // attribute 0. No particular reason for 0, but must match the layout in the shader.
			3,                  // size
			GL_FLOAT,           // type
			GL_FALSE,           // normalized?
			0,                  // stride
			(void*)0            // array buffer offset
		);

		// Draw the triangles !
	glDrawArrays(GL_TRIANGLES, 0, 6); // 2*3 indices starting at 0 -> 2 triangles

	glDisableVertexAttribArray(0);

	
	
	//Dritter Durchlauf--------------------------------------------------------------------------------------------------------

	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);

	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);
	
	//Zuerst der horizontale Gauss filter
	glDrawBuffer(GL_COLOR_ATTACHMENT0);

	gaussHorizontalShader->useShader();
	
	//Hier gebe ich die Alphatextur in die color Map
	GLuint aplhatexID3 = glGetUniformLocation(gaussHorizontalShader->programHandle, "colorMap");
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, gaussFinalTexturId);
	glUniform1i(aplhatexID3 , 0);
	
	// 1rst attribute buffer : vertices
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
		glVertexAttribPointer(
			0,                  // attribute 0. No particular reason for 0, but must match the layout in the shader.
			3,                  // size
			GL_FLOAT,           // type
			GL_FALSE,           // normalized?
			0,                  // stride
			(void*)0            // array buffer offset
		);

		// Draw the triangles !
	glDrawArrays(GL_TRIANGLES, 0, 6); // 2*3 indices starting at 0 -> 2 triangles

	glDisableVertexAttribArray(0);
	
	//Jetzt wird noch der vertikale Gauss Filter angewandt-----------------------------------------------------------
	
	glBindFramebuffer(GL_FRAMEBUFFER, fboGaussId);
	glViewport(0,0,WIDTH, HEIGHT);
	
	//Zuerst der horizontale Gauss filter
	glDrawBuffer(GL_COLOR_ATTACHMENT1);

	gaussFinalShader->useShader();
	
	//Hier gebe ich die Alphatextur in die color Map
	GLuint FinalaplhatexID3 = glGetUniformLocation(gaussFinalShader->programHandle, "colorMap");
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, gaussHorizontalTexturId);
	glUniform1i(FinalaplhatexID3 , 0);

	// 1rst attribute buffer : vertices
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
		glVertexAttribPointer(
			0,                  // attribute 0. No particular reason for 0, but must match the layout in the shader.
			3,                  // size
			GL_FLOAT,           // type
			GL_FALSE,           // normalized?
			0,                  // stride
			(void*)0            // array buffer offset
		);

		// Draw the triangles !
	glDrawArrays(GL_TRIANGLES, 0, 6); // 2*3 indices starting at 0 -> 2 triangles

	glDisableVertexAttribArray(0);
	
}

void Fbo::renderOnScreen(Shader* textureShader){

	glBindFramebuffer(GL_FRAMEBUFFER,0);
	glViewport(0,0, WIDTH, HEIGHT);

	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	textureShader->useShader();
	//Bind first texture
	GLuint texID = glGetUniformLocation(textureShader->programHandle, "renderedTexture");
	GLuint tex2ID = glGetUniformLocation(textureShader->programHandle, "gaussianTexture");
	
	textureShader->useShader();
	glActiveTexture(GL_TEXTURE0);
	//HIER DIE TEXTUR, DIE AM BILDSCHIRM GERENDERT WERDEN SOLL
	glBindTexture(GL_TEXTURE_2D, textureId);
	glUniform1i(texID, 0);

	//Bind second texture
	
	glActiveTexture(GL_TEXTURE1);
	//HIER DIE TEXTUR, DIE AM BILDSCHIRM GERENDERT WERDEN SOLL
	glBindTexture(GL_TEXTURE_2D, gaussFinalTexturId);
	glUniform1i(tex2ID, 1);
	
	// 1rst attribute buffer : vertices
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
		glVertexAttribPointer(
			0,                  // attribute 0. No particular reason for 0, but must match the layout in the shader.
			3,                  // size
			GL_FLOAT,           // type
			GL_FALSE,           // normalized?
			0,                  // stride
			(void*)0            // array buffer offset
		);

		// Draw the triangles !
		glDrawArrays(GL_TRIANGLES, 0, 6); // 2*3 indices starting at 0 -> 2 triangles

		glDisableVertexAttribArray(0);

}

void Fbo::endGame(Shader* endGameShader, Texture* tex){
	glBindFramebuffer(GL_FRAMEBUFFER,0);
	glViewport(0,0, WIDTH, HEIGHT);
	
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	endGameShader->useShader();
	tex->Bind(0);
	glUniform1i(glGetUniformLocation(endGameShader->programHandle, "renderedTexture"), 0);
	
	
	// 1rst attribute buffer : vertices
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
		glVertexAttribPointer(
			0,                  // attribute 0. No particular reason for 0, but must match the layout in the shader.
			3,                  // size
			GL_FLOAT,           // type
			GL_FALSE,           // normalized?
			0,                  // stride
			(void*)0            // array buffer offset
		);

		// Draw the triangles !
		glDrawArrays(GL_TRIANGLES, 0, 6); // 2*3 indices starting at 0 -> 2 triangles

		glDisableVertexAttribArray(0);


}


Fbo::~Fbo(void){
	//delete fbo
	glDeleteFramebuffers(1,&fboId);
	//delete renderbuffer
//	glDeleteRenderbuffers(1,&rboId);
	//delete Texture
	glDeleteTextures(1,&textureId);

}

GLuint Fbo::getFbo(){
return fboId;
}