#include "Ground.h"
#include <iostream>

#include "../Info.h"
// Include GLM
#include <glm/glm.hpp>
#include <glm/gtc/type_ptr.hpp>



using namespace cgue::thewall::scene;

using namespace cgue::thewall;

Ground::Ground(glm::mat4& matrix, Shader* _shader) : SceneObject(matrix), shader(_shader)
{
		std::vector<glm::vec3> in_vertices;
		std::vector<glm::vec2> in_uvs;
		std::vector<glm::vec3> in_normals;
		std::vector<glm::vec3> in_tangents;
		std::vector<glm::vec3> in_bitangents;
		loadOBJ(ORDNER + std::string("Objects/ground.obj"), in_vertices, in_uvs, in_normals);
		computeTangentBasis(
			in_vertices, in_uvs, in_normals, // input
			in_tangents, in_bitangents    // output
		);
		indexVBO_TBN(
			in_vertices, in_uvs, in_normals, in_tangents, in_bitangents, 
			indices, vertices, uvs, normals, tangents, bitangents
		);

		// Load data to buffer
		glGenBuffers(1, &positionBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
		glBufferData(GL_ARRAY_BUFFER, vertices.size() * sizeof(glm::vec3), &vertices[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		glGenBuffers(1, &uvBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
		glBufferData(GL_ARRAY_BUFFER, uvs.size() * sizeof(glm::vec2), &uvs[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		glGenBuffers(1, &normalBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
		glBufferData(GL_ARRAY_BUFFER, normals.size() * sizeof(glm::vec3), &normals[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		glGenBuffers(1, &tangentBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, tangentBuffer);
		glBufferData(GL_ARRAY_BUFFER, tangents.size() * sizeof(glm::vec3), &tangents[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		glGenBuffers(1, &bitangentBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, bitangentBuffer);
		glBufferData(GL_ARRAY_BUFFER, bitangents.size() * sizeof(glm::vec3), &bitangents[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		// Generate a buffer for the indices as well
		glGenBuffers(1, &indexBuffer);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(unsigned short), &indices[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		// Generate VAO
		glGenVertexArrays(1, &vao);
		glBindVertexArray(vao);

		//Buffer binden

		//position
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER,positionBuffer);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, 0);

		//uv
		glEnableVertexAttribArray(1);
		glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
		glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, 0);

		//normal
		glEnableVertexAttribArray(2);
		glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
		glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, 0, 0);

		//tangent
		glEnableVertexAttribArray(3);
		glBindBuffer(GL_ARRAY_BUFFER,tangentBuffer);
		glVertexAttribPointer(3, 3, GL_FLOAT, GL_FALSE, 0, 0);
		
		//bitangent
		glEnableVertexAttribArray(4);
		glBindBuffer(GL_ARRAY_BUFFER,bitangentBuffer);
		glVertexAttribPointer(4, 3, GL_FLOAT, GL_FALSE, 0, 0);

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,indexBuffer);

		glBindVertexArray(0);
		

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
		

		glBindBuffer(GL_ARRAY_BUFFER,0);
		

		setSurfaceParameters();

		modelMatrix -= glm::mat4(glm::vec4(0.0f, 0.0f, 0.0f, 0.0f),
		glm::vec4(0.0f, 0.0f, 0.0f, 0.0f),
		glm::vec4(0.0f, 0.0f, 0.0f, 0.0f),
		glm::vec4(0.0f, 5.0f, 0.0f, 0.0f));
		text = new Texture(ORDNER + std::string("Texturen/ground.jpg"));
		textNorm = new Texture(ORDNER + std::string("Texturen/groundNormal.jpg"));
		wall = new Wall(matrix, _shader);
}

void Ground::setSurfaceParameters(){

//Specular Component festlegen
	GLuint loc = glGetUniformLocation(shader->programHandle, "materialSpecularColor");
	
	glm::vec3 materialSpecularColor = glm::vec3(0.0f,0.0f,0.0f);
	glUniform3f(loc,  materialSpecularColor.x, materialSpecularColor.y, materialSpecularColor.z );
}

Ground::~Ground(void)
{
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteBuffers(1, &tangentBuffer);
	glDeleteBuffers(1, &bitangentBuffer);
	glDeleteBuffers(1, &indexBuffer);
	glDeleteBuffers(1, &vao);
	delete wall;
	delete text;
	delete textNorm;
}

void Ground::update(float deltaT){}


void Ground::draw(){
	text->Bind(0);
	textNorm->Bind(2);
	glUniform1i(glGetUniformLocation(shader->programHandle, "colorTexture"), 0);
	glUniform1i(glGetUniformLocation(shader->programHandle, "normalTexture"), 2);
	glBindVertexArray(vao);
	glDrawElements(
			GL_TRIANGLES,      // mode
			indices.size(),    // count
			GL_UNSIGNED_SHORT, // type
			0           // element array buffer offset
		);
	glBindVertexArray(0);
	glUniformMatrix4fv(glGetUniformLocation(shader->programHandle,"modelMatrix"), 1,
		GL_FALSE, glm::value_ptr(wall->modelMatrix));
	wall->draw();
}

void Ground::setModelMatrix(btRigidBody* GroundRigidBody){
	modelMatrix[3][0] = GroundRigidBody->getWorldTransform().getOrigin().x();
	modelMatrix[3][1] = GroundRigidBody->getWorldTransform().getOrigin().y();
	modelMatrix[3][2] = GroundRigidBody->getWorldTransform().getOrigin().z();
}

void Ground::drawDepth(){
	vaoDepth(positionBuffer, indexBuffer, indices);
}


float Ground::findHigh(float x, float z){
	int pos [4] = {-1, -1, -1, -1};
	int eing = 0;
	for(int j = 0; j < 4; j++){
		int minpos = eing;
		float min = sqrt(pow((vertices[eing].x - x),2) + pow((vertices[eing].z - z),2));
		for(int i = eing + 1; i < vertices.size(); i++){
			float d = sqrt(pow((vertices[i].x - x),2) + pow((vertices[i].z - z),2));
			if(d < min && (i != pos[0] && i != pos[1] && i != pos[2] && i != pos[3])){
				if(pos[0] > -1 && vertices[i].x == vertices[pos[0]].x && vertices[i].z == vertices[pos[0]].z)
					continue;
				if(pos[1] > -1 && vertices[i].x == vertices[pos[1]].x && vertices[i].z == vertices[pos[1]].z)
					continue;
				if(pos[2] > -1 && vertices[i].x == vertices[pos[2]].x && vertices[i].z == vertices[pos[2]].z)
					continue;
				if(pos[3] > -1 && vertices[i].x == vertices[pos[3]].x && vertices[i].z == vertices[pos[3]].z)
					continue;
				minpos = i;
				min = d;
			}
		}
		pos[j] = minpos;
		if(minpos == eing)
			eing++;
	}

	float dist = 0.0f;
	dist += sqrt(pow((vertices[pos[0]].x - x),2) + pow((vertices[pos[0]].z - z),2));
	dist += sqrt(pow((vertices[pos[1]].x - x),2) + pow((vertices[pos[1]].z - z),2));
	dist += sqrt(pow((vertices[pos[2]].x - x),2) + pow((vertices[pos[2]].z - z),2));
	dist += sqrt(pow((vertices[pos[3]].x - x),2) + pow((vertices[pos[3]].z - z),2));

	dist /= 3.0f;

	float y = 0.0f;
	y += (1 - sqrt(pow(vertices[pos[0]].x - x, 2) + pow(vertices[pos[0]].z - z, 2))/dist) * vertices[pos[0]].y;
	y += (1 - sqrt(pow(vertices[pos[1]].x - x, 2) + pow(vertices[pos[1]].z - z, 2))/dist) * vertices[pos[1]].y;
	y += (1 - sqrt(pow(vertices[pos[2]].x - x, 2) + pow(vertices[pos[2]].z - z, 2))/dist) * vertices[pos[2]].y;
	y += (1 - sqrt(pow(vertices[pos[3]].x - x, 2) + pow(vertices[pos[3]].z - z, 2))/dist) * vertices[pos[3]].y;

	
	return y - 1;
}

void Ground::generateCollisionShape(){

	btTriangleMesh* trimesh = new btTriangleMesh();
	
	for(int i = 0; i+2<vertices.size(); i=i+3){
	
		btVector3 vertex0 = btVector3(vertices[i].x, vertices[i].y, vertices[i].z);
		btVector3 vertex1 = btVector3(vertices[i+1].x, vertices[i+1].y, vertices[i+1].z);
		btVector3 vertex2 = btVector3(vertices[i+2].x, vertices[i+2].y, vertices[i+2].z);
		
		trimesh->addTriangle(vertex2, vertex1, vertex0);
		
	}
	collisionShape = new btBvhTriangleMeshShape(trimesh, true);
	
}