#include "Key.h"
#include <iostream>

#include "../Info.h"
// Include GLM
#include <glm/glm.hpp>
#include <glm/gtc/type_ptr.hpp>
#include <glm/gtc/matrix_transform.hpp> 


using namespace cgue::thewall::scene;

using namespace cgue::thewall;

std::vector< glm::vec3 > Key::vertices = std::vector< glm::vec3 >();
std::vector< glm::vec2 > Key::uvs = std::vector< glm::vec2 >();
std::vector< glm::vec3 > Key::normals = std::vector< glm::vec3 >();
std::vector<unsigned short> Key::indices = std::vector<unsigned short>();
GLuint Key::positionBuffer = 0;
GLuint Key::normalBuffer = 0;
GLuint Key::uvBuffer = 0;
GLuint Key::indexBuffer = 0;
GLuint Key::vao = 0;

Key::Key(glm::mat4& matrix, Shader* _shader) : SceneObject(matrix), shader(_shader)
{
	if(vertices.size() == 0){
		std::vector<glm::vec3> in_vertices;
		std::vector<glm::vec2> in_uvs;
		std::vector<glm::vec3> in_normals;
		loadOBJ(ORDNER + std::string("Objects/key.obj"), in_vertices, in_uvs, in_normals);
		indexVBO(in_vertices,in_uvs,in_normals,indices,vertices,uvs,normals);

		// Load data to buffer
		glGenBuffers(1, &positionBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
		glBufferData(GL_ARRAY_BUFFER, vertices.size() * sizeof(glm::vec3), &vertices[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		/*glGenBuffers(1, &normalBuffer);
		glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
		glBufferData(GL_ARRAY_BUFFER, normals.size() * sizeof(glm::vec3), &normals[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);*/

		glGenBuffers(1, &uvBuffer);
		glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
		glBufferData(GL_ARRAY_BUFFER, uvs.size() * sizeof(glm::vec2), &uvs[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		glGenBuffers(1, &indexBuffer);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(unsigned short), &indices[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		// Generate VAO
		glGenVertexArrays(1, &vao);
		glBindVertexArray(vao);

		//Buffer binden
		glBindBuffer(GL_ARRAY_BUFFER,positionBuffer);
		GLint positionIndex = glGetAttribLocation(shader->programHandle, "position");
		glEnableVertexAttribArray(positionIndex);
		glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

		/*glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
		GLint normalIndex = glGetAttribLocation(shader->programHandle, "normal");
		glEnableVertexAttribArray(normalIndex);
		glVertexAttribPointer(normalIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);*/

		glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
		GLint uvIndex = glGetAttribLocation(shader->programHandle, "uv");
		glEnableVertexAttribArray(uvIndex);
		glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,indexBuffer);

		glBindVertexArray(0);


		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
		glBindBuffer(GL_ARRAY_BUFFER,0);
		setSurfaceParameters();
	}

	setSurfaceParameters();
	modelMatrix = glm::scale(modelMatrix, glm::vec3(0.1f,0.1f,0.1f));
}

void Key::setSurfaceParameters(){

//Specular Component festlegen
	GLuint loc = glGetUniformLocation(shader->programHandle, "materialSpecularColor");
	
	glm::vec3 materialSpecularColor = glm::vec3(0.0f,0.0f,0.0f);
	glUniform3f(loc,  materialSpecularColor.x, materialSpecularColor.y, materialSpecularColor.z );
	

}

Key::~Key(void)
{
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteBuffers(1, &vao);
}


void Key::update(float deltaT){}


void Key::draw(){
	glBindVertexArray(vao);

	glDrawArrays(GL_TRIANGLES, 0, indices.size());
	
	glBindVertexArray(0);
}

void Key::setModelMatrix(btRigidBody* keyRigidBody){
	//modelMatrix[3][0]  = keyRigidBody->getWorldTransform().getOrigin().x();
	//modelMatrix[3][1]  = keyRigidBody->getWorldTransform().getOrigin().y()-1;
	//modelMatrix[3][2]  = keyRigidBody->getWorldTransform().getOrigin().z();
	int pos[5] = {2, 35, 28, 110, 212};
}

void Key::drawDepth(){
	vaoDepth(positionBuffer, indexBuffer, indices);
}

