#include "Raven.h"
#include <iostream>

#include "../Info.h"
#include "../Texture.h"
#include <time.h>

using namespace cgue::thewall::scene;

using namespace cgue::thewall;


Raven::Raven(glm::mat4& matrix, Shader* _shader, int _x, int _z) : SceneObject(matrix), shader(_shader)
{
	std::vector<glm::vec3> in_vertices;
	std::vector<glm::vec2> in_uvs;
	std::vector<glm::vec3> in_normals;
	bool res = loadOBJ(ORDNER + std::string("Objects/ravenbd.obj"), in_vertices, in_uvs, in_normals);
	indexVBO(in_vertices,in_uvs,in_normals,indices,vertices,uvs,normals);

	wings[0] = new Wing(matrix, shader, ORDNER + std::string("Objects/ravenrw.obj"));
	wings[1] = new Wing(matrix, shader, ORDNER + std::string("Objects/ravenlw.obj"));
	wings[1]->mirrored = true;

//	texture = new Texture(ORDNER + std::string("Texturen/down.jpg"));
	texture = new Texture(ORDNER + std::string("Texturen/key.jpg"));

	wings[0]->setFly(true);
	wings[1]->setFly(true);
	// Load data to buffer
	glGenBuffers(1, &positionBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	glBufferData(GL_ARRAY_BUFFER, vertices.size() * sizeof(glm::vec3), &vertices[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &normalBuffer);
	glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
	glBufferData(GL_ARRAY_BUFFER, normals.size() * sizeof(glm::vec3), &normals[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &uvBuffer);
	glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
	glBufferData(GL_ARRAY_BUFFER, uvs.size() * sizeof(glm::vec2), &uvs[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &indexBuffer);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(unsigned short), &indices[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// Generate VAO
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	//Buffer binden
	glBindBuffer(GL_ARRAY_BUFFER,positionBuffer);
	GLint positionIndex = glGetAttribLocation(shader->programHandle, "position");
	glEnableVertexAttribArray(positionIndex);
	glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
	GLint normalIndex = glGetAttribLocation(shader->programHandle, "normal");
	glEnableVertexAttribArray(normalIndex);
	glVertexAttribPointer(normalIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
	GLint uvIndex = glGetAttribLocation(shader->programHandle, "uv");
	glEnableVertexAttribArray(uvIndex);
	glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,indexBuffer);

	glBindVertexArray(0);


	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
	glBindBuffer(GL_ARRAY_BUFFER,0);


	x = _x;
	z = _z;
	//fly = false;
	direction = glm::vec3(1,0,0);
	setSurfaceParameters();
	modelMatrix = glm::scale(modelMatrix, glm::vec3(0.1f,0.1f,0.1f));
	modelMatrix = glm::rotate(modelMatrix, 90.0f, glm::vec3(0,1,0));
	time_t t;

    time(&t);
    srand((unsigned int)t); 

	phi = rand()%360;
}


void Raven::setSurfaceParameters(){

//Specular Component festlegen
	GLuint loc = glGetUniformLocation(shader->programHandle, "materialSpecularColor");
	
	glm::vec3 materialSpecularColor = glm::vec3(0.1f,0.1f,0.1f);
	glUniform3f(loc,  materialSpecularColor.x, materialSpecularColor.y, materialSpecularColor.z );
	

}
Raven::~Raven(void)
{
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteBuffers(1, &vao);
	delete wings[0];
	delete wings[1];
}

bool Raven::getFly(){return fly;}

void Raven::update(float deltaT){
wings[0]->setFly(true);
wings[1]->setFly(true);
wings[0]->update(deltaT);
wings[1]->update(deltaT);
}


void Raven::draw(){

	texture->Bind(0);

	shader->useShader();

	glUniform1i(glGetUniformLocation(shader->programHandle, "colorTexture"), 0);

	glUniformMatrix4fv(glGetUniformLocation(shader->programHandle,"modelMatrix"), 1,
		GL_FALSE, glm::value_ptr(modelMatrix));

	

	glBindVertexArray(vao);

	
	glDrawArrays(GL_TRIANGLES, 0, indices.size());

	
	glBindVertexArray(0);


	
	glm::mat4 mm = wings[0]->modelMatrix;
	mm[3] = modelMatrix[3];

	
	glUniform1i(glGetUniformLocation(shader->programHandle, "colorTexture"), 0);

	
	glUniformMatrix4fv(glGetUniformLocation(shader->programHandle,"modelMatrix"), 1,
		GL_FALSE, glm::value_ptr(mm));
	
	wings[0]->draw();

	
	mm = wings[1]->modelMatrix;
	mm[3] = modelMatrix[3];
	glUniform1i(glGetUniformLocation(shader->programHandle, "colorTexture"), 0);

	
	glUniformMatrix4fv(glGetUniformLocation(shader->programHandle,"modelMatrix"), 1,
		GL_FALSE, glm::value_ptr(mm));

	
	wings[1]->draw();
}

void Raven::setFly(bool b){
	fly = b;
	wings[0]->setFly(b);
	wings[1]->setFly(b);
}

void Raven::setModelMatrix(glm::mat4 mm){
	modelMatrix = mm;
}

void Raven::setUp(glm::vec3 u){
	up = u;
}

void Raven::drawDepth(){
	vaoDepth(positionBuffer, indexBuffer, indices);
	wings[0]->drawDepth();
	wings[1]->drawDepth();
}

void Raven::setModelMatrix(btRigidBody* ravenRigidBody){
	modelMatrix[3][0] = ravenRigidBody->getWorldTransform().getOrigin().x();
	modelMatrix[3][1] = ravenRigidBody->getWorldTransform().getOrigin().y();
	modelMatrix[3][2] = ravenRigidBody->getWorldTransform().getOrigin().z();
}