#include "Surrounding.h"
#include "bullet-2.81-rev2613\src\BulletCollision\CollisionShapes\btStridingMeshInterface.h"
#include "../Info.h"
#include <math.h> 
#include <iostream>

using namespace cgue::thewall::scene;

using namespace cgue::thewall;

Surrounding::Surrounding(glm::mat4& matrix, Shader* _shader) : SceneObject(matrix), shader(_shader)
{
	bool res = loadOBJ(ORDNER + std::string("Objects/OLD/ground.obj"), vertices, uvs, normals);
	// Load data to buffer
	glGenBuffers(1, &positionBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	glBufferData(GL_ARRAY_BUFFER, vertices.size() * sizeof(glm::vec3), &vertices[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &normalBuffer);
	glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
	glBufferData(GL_ARRAY_BUFFER, normals.size() * sizeof(glm::vec3), &normals[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &uvBuffer);
	glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
	glBufferData(GL_ARRAY_BUFFER, uvs.size() * sizeof(glm::vec2), &uvs[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// Generate VAO
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	//Buffer binden
	glBindBuffer(GL_ARRAY_BUFFER,positionBuffer);
	GLint positionIndex = glGetAttribLocation(shader->programHandle, "position");
	glEnableVertexAttribArray(positionIndex);
	glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
	GLint normalIndex = glGetAttribLocation(shader->programHandle, "normal");
	glEnableVertexAttribArray(normalIndex);
	glVertexAttribPointer(normalIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
	GLint uvIndex = glGetAttribLocation(shader->programHandle, "uv");
	glEnableVertexAttribArray(uvIndex);
	glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);

	glBindVertexArray(0);


	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
	glBindBuffer(GL_ARRAY_BUFFER,0);

	modelMatrix -= glm::mat4(glm::vec4(0.0f, 0.0f, 0.0f, 0.0f),
			glm::vec4(0.0f, 0.0f, 0.0f, 0.0f),
			glm::vec4(0.0f, 0.0f, 0.0f, 0.0f),
			glm::vec4(0.0f, 5.0f, 0.0f, 0.0f));
	
	generateCollisionShape();
}


Surrounding::~Surrounding(void)
{
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteBuffers(1, &vao);
}

void Surrounding::draw(){
	
}

void Surrounding::update(float deltaT){}

void Surrounding::setSurfaceParameters(){

}

float Surrounding::findHigh(float x, float z){
	int pos [4] = {-1, -1, -1, -1};
	int eing = 0;
	for(int j = 0; j < 4; j++){
		int minpos = eing;
		float min = sqrt(pow((vertices[eing].x - x),2) + pow((vertices[eing].z - z),2));
		for(int i = eing + 1; i < vertices.size(); i++){
			float d = sqrt(pow((vertices[i].x - x),2) + pow((vertices[i].z - z),2));
			if(d < min && (i != pos[0] && i != pos[1] && i != pos[2] && i != pos[3])){
				if(pos[0] > -1 && vertices[i].x == vertices[pos[0]].x && vertices[i].z == vertices[pos[0]].z)
					continue;
				if(pos[1] > -1 && vertices[i].x == vertices[pos[1]].x && vertices[i].z == vertices[pos[1]].z)
					continue;
				if(pos[2] > -1 && vertices[i].x == vertices[pos[2]].x && vertices[i].z == vertices[pos[2]].z)
					continue;
				if(pos[3] > -1 && vertices[i].x == vertices[pos[3]].x && vertices[i].z == vertices[pos[3]].z)
					continue;
				minpos = i;
				min = d;
			}
		}
		pos[j] = minpos;
		if(minpos == eing)
			eing++;
	}

	float dist = 0.0f;
	dist += sqrt(pow((vertices[pos[0]].x - x),2) + pow((vertices[pos[0]].z - z),2));
	dist += sqrt(pow((vertices[pos[1]].x - x),2) + pow((vertices[pos[1]].z - z),2));
	dist += sqrt(pow((vertices[pos[2]].x - x),2) + pow((vertices[pos[2]].z - z),2));
	dist += sqrt(pow((vertices[pos[3]].x - x),2) + pow((vertices[pos[3]].z - z),2));

	dist /= 3.0f;

	float y = 0.0f;
	y += (1 - sqrt(pow(vertices[pos[0]].x - x, 2) + pow(vertices[pos[0]].z - z, 2))/dist) * vertices[pos[0]].y;
	y += (1 - sqrt(pow(vertices[pos[1]].x - x, 2) + pow(vertices[pos[1]].z - z, 2))/dist) * vertices[pos[1]].y;
	y += (1 - sqrt(pow(vertices[pos[2]].x - x, 2) + pow(vertices[pos[2]].z - z, 2))/dist) * vertices[pos[2]].y;
	y += (1 - sqrt(pow(vertices[pos[3]].x - x, 2) + pow(vertices[pos[3]].z - z, 2))/dist) * vertices[pos[3]].y;

	return y - 1;
}

void Surrounding::generateCollisionShape(){

	btTriangleMesh* trimesh = new btTriangleMesh();
	
	for(int i = 0; i+2<vertices.size(); i=i+3){
	
		btVector3 vertex0 = btVector3(vertices[i].x, vertices[i].y, vertices[i].z);
		btVector3 vertex1 = btVector3(vertices[i+1].x, vertices[i+1].y, vertices[i+1].z);
		btVector3 vertex2 = btVector3(vertices[i+2].x, vertices[i+2].y, vertices[i+2].z);
		
		trimesh->addTriangle(vertex2, vertex1, vertex0);
		
	}
	collisionShape = new btBvhTriangleMeshShape(trimesh, true);
	
}


void Surrounding::setModelMatrix(btRigidBody* groundRigidBody){
modelMatrix[3][0] = groundRigidBody->getWorldTransform().getOrigin().x();
modelMatrix[3][1] = groundRigidBody->getWorldTransform().getOrigin().y();
modelMatrix[3][2] = groundRigidBody->getWorldTransform().getOrigin().z();
}