#include "Tree.h"
#include <iostream>

#include "../Info.h"
// Include GLM
#include <glm/glm.hpp>
#include <glm/gtc/type_ptr.hpp>



using namespace cgue::thewall::scene;

using namespace cgue::thewall;

std::vector< glm::vec3 > Tree::vertices = std::vector< glm::vec3 >();
std::vector< glm::vec2 > Tree::uvs = std::vector< glm::vec2 >();
std::vector< glm::vec3 > Tree::normals = std::vector< glm::vec3 >();
std::vector<glm::vec3> Tree::tangents = std::vector< glm::vec3 >();
std::vector<glm::vec3> Tree::bitangents = std::vector< glm::vec3 >();
std::vector<unsigned short> Tree::indices = std::vector<unsigned short>();

GLuint Tree::positionBuffer = 0;
GLuint Tree::normalBuffer = 0;
GLuint Tree::uvBuffer = 0;
GLuint Tree::indexBuffer = 0;
GLuint Tree::tangentBuffer = 0;
GLuint Tree::bitangentBuffer = 0;
GLuint Tree::vao = 0;

Tree::Tree(glm::mat4& matrix, Shader* _shader) : SceneObject(matrix), shader(_shader)
{
	if(vertices.size() == 0){
		std::vector<glm::vec3> in_vertices;
		std::vector<glm::vec2> in_uvs;
		std::vector<glm::vec3> in_normals;
		std::vector<glm::vec3> in_tangents;
		std::vector<glm::vec3> in_bitangents;
		loadOBJ(ORDNER + std::string("Objects/tree.obj"), in_vertices, in_uvs, in_normals);
		computeTangentBasis(
			in_vertices, in_uvs, in_normals, // input
			in_tangents, in_bitangents    // output
		);
		indexVBO_TBN(
			in_vertices, in_uvs, in_normals, in_tangents, in_bitangents, 
			indices, vertices, uvs, normals, tangents, bitangents
		);

		// Load data to buffer
		glGenBuffers(1, &positionBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
		glBufferData(GL_ARRAY_BUFFER, vertices.size() * sizeof(glm::vec3), &vertices[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		glGenBuffers(1, &uvBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
		glBufferData(GL_ARRAY_BUFFER, uvs.size() * sizeof(glm::vec2), &uvs[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		glGenBuffers(1, &normalBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
		glBufferData(GL_ARRAY_BUFFER, normals.size() * sizeof(glm::vec3), &normals[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		glGenBuffers(1, &tangentBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, tangentBuffer);
		glBufferData(GL_ARRAY_BUFFER, tangents.size() * sizeof(glm::vec3), &tangents[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		glGenBuffers(1, &bitangentBuffer);
		glBindBuffer(GL_ARRAY_BUFFER, bitangentBuffer);
		glBufferData(GL_ARRAY_BUFFER, bitangents.size() * sizeof(glm::vec3), &bitangents[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		// Generate a buffer for the indices as well
		glGenBuffers(1, &indexBuffer);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(unsigned short), &indices[0], GL_STATIC_DRAW);
		glBindBuffer(GL_ARRAY_BUFFER, 0);

		// Generate VAO
		glGenVertexArrays(1, &vao);
		glBindVertexArray(vao);

		//Buffer binden

		//position
		glEnableVertexAttribArray(0);
		glBindBuffer(GL_ARRAY_BUFFER,positionBuffer);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, 0);

		//uv
		glEnableVertexAttribArray(1);
		glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
		glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, 0);

		//normal
		glEnableVertexAttribArray(2);
		glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
		glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, 0, 0);

		//tangent
		glEnableVertexAttribArray(3);
		glBindBuffer(GL_ARRAY_BUFFER,tangentBuffer);
		glVertexAttribPointer(3, 3, GL_FLOAT, GL_FALSE, 0, 0);
		
		//bitangent
		glEnableVertexAttribArray(4);
		glBindBuffer(GL_ARRAY_BUFFER,bitangentBuffer);
		glVertexAttribPointer(4, 3, GL_FLOAT, GL_FALSE, 0, 0);

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,indexBuffer);

		glBindVertexArray(0);
		

		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
		

		glBindBuffer(GL_ARRAY_BUFFER,0);
		

		setSurfaceParameters();

	}
}

void Tree::setSurfaceParameters(){

//Specular Component festlegen
	GLuint loc = glGetUniformLocation(shader->programHandle, "materialSpecularColor");
	
	glm::vec3 materialSpecularColor = glm::vec3(0.0f,0.0f,0.0f);
	glUniform3f(loc,  materialSpecularColor.x, materialSpecularColor.y, materialSpecularColor.z );
}

Tree::~Tree(void)
{
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteBuffers(1, &tangentBuffer);
	glDeleteBuffers(1, &bitangentBuffer);
	glDeleteBuffers(1, &indexBuffer);
	glDeleteBuffers(1, &vao);
}

void Tree::update(float deltaT){}


void Tree::draw(){
	glBindVertexArray(vao);
	glDrawElements(
			GL_TRIANGLES,      // mode
			indices.size(),    // count
			GL_UNSIGNED_SHORT, // type
			0           // element array buffer offset
		);
	glBindVertexArray(0);
}

void Tree::setModelMatrix(btRigidBody* treeRigidBody){
	modelMatrix[3][0] = treeRigidBody->getWorldTransform().getOrigin().x();
	modelMatrix[3][1] = treeRigidBody->getWorldTransform().getOrigin().y()-5;
	modelMatrix[3][2] = treeRigidBody->getWorldTransform().getOrigin().z();
}

void Tree::drawDepth(){
	vaoDepth(positionBuffer, indexBuffer, indices);
}