#include "Triangle.h"
#include <glm/gtc/matrix_transform.hpp>

using namespace cgue::thewall;
using namespace cgue::thewall::scene;

Triangle::Triangle(glm::mat4& matrix, Shader* _shader) : SceneObject(matrix), shader(_shader)
{
	// Load data to buffer
	glGenBuffers(1, &positionBuffer);
	glBindBuffer(GL_ARRAY_BUFFER,positionBuffer);
	glBufferData(GL_ARRAY_BUFFER,TRIANGLE_VERTEX_COUNT * 3 * sizeof(float), positions, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &indexBuffer);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,indexBuffer);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER,TRIANGLE_INDEX_COUNT * sizeof(unsigned int), indices, GL_STATIC_DRAW);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	// Generate VAO
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	glBindBuffer(GL_ARRAY_BUFFER,positionBuffer);
	GLint positionIndex = glGetAttribLocation(shader->programHandle, "position");
	glEnableVertexAttribArray(positionIndex);
	glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);

	glBindVertexArray(0);


	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
	glBindBuffer(GL_ARRAY_BUFFER,0);
}
Triangle::~Triangle(void){
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &indexBuffer);
	glDeleteBuffers(1, &vao);
}

void Triangle::update(float deltaT){
}
void Triangle::draw(){
	glBindVertexArray(vao);

	glDrawElements(GL_TRIANGLES,TRIANGLE_INDEX_COUNT,GL_UNSIGNED_INT,0);
	
	glBindVertexArray(0);
}

const float Triangle::positions[TRIANGLE_VERTEX_COUNT * 3] = {

	-0.7f, -0.5f, 0.0f,
	0.7f, -0.5f, 0.0f,
	0.0f, 0.5f, 0.0f
};

const unsigned int Triangle::indices[TRIANGLE_INDEX_COUNT] = {
	0,1,2
};
