#include "Wing.h"

using namespace cgue::thewall::scene;

using namespace cgue::thewall;


Wing::Wing(glm::mat4& matrix, Shader* _shader, std::string _path) : SceneObject(matrix), shader(_shader), path(_path)
{
	std::vector<glm::vec3> in_vertices;
	std::vector<glm::vec2> in_uvs;
	std::vector<glm::vec3> in_normals;
	loadOBJ(path, in_vertices, in_uvs, in_normals);
	indexVBO(in_vertices,in_uvs,in_normals,indices,vertices,uvs,normals);

	// Load data to buffer
	glGenBuffers(1, &positionBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	glBufferData(GL_ARRAY_BUFFER, vertices.size() * sizeof(glm::vec3), &vertices[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &normalBuffer);
	glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
	glBufferData(GL_ARRAY_BUFFER, normals.size() * sizeof(glm::vec3), &normals[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &uvBuffer);
	glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
	glBufferData(GL_ARRAY_BUFFER, uvs.size() * sizeof(glm::vec2), &uvs[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &indexBuffer);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.size() * sizeof(unsigned short), &indices[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// Generate VAO
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	//Buffer binden
	glBindBuffer(GL_ARRAY_BUFFER,positionBuffer);
	GLint positionIndex = glGetAttribLocation(shader->programHandle, "position");
	glEnableVertexAttribArray(positionIndex);
	glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER,normalBuffer);
	GLint normalIndex = glGetAttribLocation(shader->programHandle, "normal");
	glEnableVertexAttribArray(normalIndex);
	glVertexAttribPointer(normalIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
	GLint uvIndex = glGetAttribLocation(shader->programHandle, "uv");
	glEnableVertexAttribArray(uvIndex);
	glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,indexBuffer);

	glBindVertexArray(0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
	glBindBuffer(GL_ARRAY_BUFFER,0);

	fly = false;
	direction = glm::vec3(0,0,1);
	setSurfaceParameters();
	modelMatrix = glm::scale(modelMatrix, glm::vec3(0.1f,0.1f,0.1f));
	modelMatrix = glm::rotate(modelMatrix, 90.0f, glm::vec3(0,1,0));
	changeWingdir = -1.0f;
	mirrored = false;
}


void Wing::setSurfaceParameters(){

//Specular Component festlegen
	GLuint loc = glGetUniformLocation(shader->programHandle, "materialSpecularColor");
	
	glm::vec3 materialSpecularColor = glm::vec3(0.1f,0.1f,0.1f);
	glUniform3f(loc,  materialSpecularColor.x, materialSpecularColor.y, materialSpecularColor.z );
	

}
Wing::~Wing(void)
{
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteBuffers(1, &vao);
}


void Wing::update(float deltaT){
	if(fly || (changeWingdir > -1.0f  && changeWingdir <= 1.0f)){
		float w = 120.0f;
		if(changeWingdir >= 0){
			modelMatrix = glm::rotate(modelMatrix, mirrored ? -w * deltaT : w * deltaT, direction);
			changeWingdir += 3 * deltaT;
			if(changeWingdir >= 1.0f)
				changeWingdir = -1.0f;
		}else{
			modelMatrix = glm::rotate(modelMatrix, mirrored ? w * deltaT : -w * deltaT, direction);
			changeWingdir += 3 * deltaT;
		}

	}else
		changeWingdir = -1.0f;
}


void Wing::draw(){
	glBindVertexArray(vao);

	glDrawArrays(GL_TRIANGLES, 0, indices.size());
	
	glBindVertexArray(0);
}

void Wing::setFly(bool b){
	fly = b;
}


void Wing::setModelMatrix(glm::mat4 mm){
	modelMatrix = mm;
}

void Wing::drawDepth(){
	vaoDepth(positionBuffer, indexBuffer, indices);
}