#define _CRT_SECURE_NO_WARNINGS

#include "Shader.h"

#include <iostream>
#include <fstream>

using namespace cgue::thewall;
using namespace std;

Shader::Shader(const string& vertexShader, const string& fragmentShader) : programHandle(0), vertexHandle(0), fragmentHandle(0)
{
	programHandle = glCreateProgram();

	if (programHandle == 0){
		cout << "Failed to create shader program" << endl;
		system("PAUSE");
		exit(-1);
	}

	// Load Shader
	loadShader(vertexShader, GL_VERTEX_SHADER, vertexHandle);
	loadShader(fragmentShader, GL_FRAGMENT_SHADER, fragmentHandle);
	geometryHandle = NULL;
	// Link Shader
	link();
}

Shader::Shader(const string& vertexShader, const string& fragmentShader, const string& geometryShader) : programHandle(0), vertexHandle(0), fragmentHandle(0), geometryHandle(0)
{
	programHandle = glCreateProgram();

	if (programHandle == 0){
		cout << "Failed to create shader program" << endl;
		system("PAUSE");
		exit(-1);
	}

	// Load Shader
	loadShader(vertexShader, GL_VERTEX_SHADER, vertexHandle);
	loadShader(fragmentShader, GL_FRAGMENT_SHADER, fragmentHandle);
	loadShader(geometryShader, GL_GEOMETRY_SHADER, geometryHandle);
	// Link Shader
	link();
}

Shader::~Shader(void){
	glDeleteProgram(programHandle);
	glDeleteShader(vertexHandle);
	glDeleteShader(fragmentHandle);
}

void Shader::useShader() const{
	glUseProgram(programHandle);
}

void Shader::loadShader(const string& shader, GLenum shaderType, GLuint& handle){

	ifstream shaderFile(shader);
	if (shaderFile.good()){
		// read file into string
		string code = string(istreambuf_iterator<char>(shaderFile), istreambuf_iterator<char>());
		shaderFile.close();

		// generate shader handle
		handle = glCreateShader(shaderType);
 
		if (handle == 0){
			cout << "Failed to create shader" << endl;
			system("PAUSE");
			exit(-1);
		}

		// compile shader
		const char* codePtr = code.c_str();
		glShaderSource(handle, 1, &codePtr, NULL);
		glCompileShader(handle);

		// Testing for compilation errors
		GLint succeded;
		glGetShaderiv(handle, GL_COMPILE_STATUS, &succeded);
 
		if (succeded == GL_FALSE || !glIsShader(handle)){
			// read log and output it to std::cout
			GLint logSize;
			glGetShaderiv(handle, GL_INFO_LOG_LENGTH, &logSize);
			auto message = new char[logSize];
 
			glGetShaderInfoLog(handle, logSize, NULL, message);
			cout << "Failed to compile shader " << shader << std::endl;
			cout << message << endl;
			system("PAUSE");
			delete[] message;
			exit(-1);
		}
	}else{
		cout << "Failed to open shader file: " << shaderFile << endl;
		system("PAUSE");
		exit(-1);
	}
}

void Shader::link(){

	// Attach shader to program
	glAttachShader(programHandle, vertexHandle);
	glAttachShader(programHandle, fragmentHandle);
	if(geometryHandle != NULL)
		glAttachShader(programHandle, geometryHandle);

	// Bind output
	glBindFragDataLocation(programHandle, 0, "fragColor");

	// Link programs
	glLinkProgram(programHandle);
 
	// check for errors
	GLint succeded;
	glGetProgramiv(programHandle, GL_LINK_STATUS, &succeded);
 
	if (!succeded){
		// output errors
		GLint logSize;
		glGetProgramiv(programHandle, GL_INFO_LOG_LENGTH, &logSize);
 
		auto message = new char[logSize];
		glGetProgramInfoLog(programHandle, logSize, NULL, message);
 
		cout << "Failed to link shader program" << endl;
		cout << message << endl;
		system("PAUSE");
 
		delete[] message;
		exit(-1);
	}
}