#version 330 core

struct SpotLight
{
	vec3 vColor;
	vec3 vPosition;
	vec3 vDirection;
  
  float bOn;

	float fConeAngle, fConeCosine;
  
  float fLinearAtt;
};

in vec3 normalCameraSpace;
in vec2 fUVs;
in vec3 lightDirectionCameraSpace;
in vec3 eyeDirectionCameraSpace;
in vec3 positionWorldSpace;

layout(location = 0)  out vec4 fragColor[2];

uniform sampler2D colorTexture;
uniform vec3 lightPositionWorldSpace;
uniform vec3 materialSpecularColor;
uniform SpotLight spotLight;
uniform float alpha;
uniform vec3 positionCamera;

vec4 GetSpotLightColor(const SpotLight spotLight, vec3 vWorldPos)
{
  if(spotLight.bOn <= 0)return vec4(0.0, 0.0, 0.0, 0.0);
	float fDistance = distance(vWorldPos, spotLight.vPosition);
  
  vec3 vDir = vWorldPos-spotLight.vPosition;
  vDir = normalize(vDir);
  
  float fCosine = dot(spotLight.vDirection, vDir);
  
  float fDif = 1.0-spotLight.fConeCosine;
  
  float fFactor = clamp((fCosine-spotLight.fConeCosine)/fDif, 0.0, 1.0);
    
  if(fCosine > spotLight.fConeCosine)
    return vec4(spotLight.vColor, 1.0)*fFactor/(fDistance*spotLight.fLinearAtt);
  
  return vec4(0.0, 0.0, 0.0, 0.0);
}

in vec4 shadowCoord;
uniform sampler2DShadow shadowMap;
vec2 poissonDisk[16] = vec2[]( 
   vec2( -0.94201624, -0.39906216 ), 
   vec2( 0.94558609, -0.76890725 ), 
   vec2( -0.094184101, -0.92938870 ), 
   vec2( 0.34495938, 0.29387760 ), 
   vec2( -0.91588581, 0.45771432 ), 
   vec2( -0.81544232, -0.87912464 ), 
   vec2( -0.38277543, 0.27676845 ), 
   vec2( 0.97484398, 0.75648379 ), 
   vec2( 0.44323325, -0.97511554 ), 
   vec2( 0.53742981, -0.47373420 ), 
   vec2( -0.26496911, -0.41893023 ), 
   vec2( 0.79197514, 0.19090188 ), 
   vec2( -0.24188840, 0.99706507 ), 
   vec2( -0.81409955, 0.91437590 ), 
   vec2( 0.19984126, 0.78641367 ), 
   vec2( 0.14383161, -0.14100790 ) 
);
float random(vec3 seed, int i){
	vec4 seed4 = vec4(seed,i);
	float dot_product = dot(seed4, vec4(12.9898,78.233,45.164,94.673));
	return fract(sin(dot_product) * 43758.5453);
}

void main(){
	vec4 spotlightColor = GetSpotLightColor(spotLight, positionWorldSpace);
		
	

	vec3 materialDiffuseColor = texture(colorTexture, fUVs).rgb; 
	
	vec3 lightColor = vec3(1.0f,1.0f,1.0f);
	float lightPower = 0.0f;

	vec3 n = normalize(normalCameraSpace);
	vec3 l = normalize(lightDirectionCameraSpace);
	float cosTheta = clamp(dot(n,l), 0.0f,1.0f);

	float distancePosFromVert = length( lightPositionWorldSpace - positionWorldSpace); 

	vec3 materialAmbientColor = vec3(0.1f, 0.1f, 0.1f)*materialDiffuseColor;

	vec3 E = normalize(eyeDirectionCameraSpace); //Eye vector(towards the camera)
	vec3 R = reflect(-l,n);//Direction in which the triangle reflects the light
	float cosAlpha = clamp(dot(E,R),0,1); //Cosine of the angle between the Eye vector and the Reflect vector

	vec3 specular = materialSpecularColor*lightColor*lightPower*pow(cosAlpha,5);
	vec3 diffuse = materialDiffuseColor*lightColor*lightPower*cosTheta/(distancePosFromVert*distancePosFromVert);

	float visibility=1.0;
	float bias = 0.005*tan(acos(cosTheta));
	bias = clamp(bias, 0,0.01);
	for (int i=0;i<4;i++){
		//int index = int(16.0*random(floor(positionWorldSpace.xyz*1000.0), i))%16;
		//visibility -= 0.2*(1.0-texture( shadowMap, vec3(shadowCoord.xy + poissonDisk[index]/5000.0,  (shadowCoord.z-bias)/shadowCoord.w) ));
		//int index = int(16.0*random(floor(positionWorldSpace.xyz*1000.0), i))%16; 
		visibility -= 0.2*(1.0-texture( shadowMap, vec3(shadowCoord.xy, (shadowCoord.z-bias)/shadowCoord.w) ));
	}

	//for (int i=0;i<4;i++){
	//  if ( texture2D( shadowMap, vec2(shadowCoord.xy + poissonDisk[i]/700.0) ).z  <  shadowCoord.z-bias ){
	//	visibility-=0.2;
	//  }
	//}

	float fog = exp(-0.04*length(positionCamera - positionWorldSpace));
		
	fog = 1.0-clamp(fog, 0.0, 1.0);

	fragColor[0] = vec4(diffuse+specular+materialAmbientColor, 1.0) * vec4(visibility) + vec4(diffuse+specular+materialAmbientColor, 1.0)*spotlightColor * spotLight.bOn;
	fragColor[1] = vec4(materialDiffuseColor*alpha,0.0);

	fragColor[0] = mix(fragColor[0], vec4(0.2f, 0.2f, 0.2f, 1.0f), fog);
}

