#version 330 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec2 uv;
layout(location = 2) in vec3 normal;
layout(location = 3) in vec3 tangent;
layout(location = 4) in vec3 bitangent;

out vec2 fUVs;
out vec3 lightDirectionCameraSpace;
out vec3 eyeDirectionCameraSpace;
out vec3 positionWorldSpace;

out vec3 lightDirectionTangentspace;
out vec3 eyeDirectionTangentspace;

uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform mat4 modelMatrix;
uniform vec3 lightPositionWorldSpace;
uniform mat4 PVM;//ProjectionViewModel MAtrix
uniform mat4 VM;//ViewModel Matrix

out vec4 shadowCoord;
uniform mat4 depthBiasVP;

void main(){
	mat4 depthBiasMVP = depthBiasVP * modelMatrix;
	shadowCoord = depthBiasMVP * vec4(position,1);

	fUVs = uv;

	
	mat3 mv3x3 = mat3( VM[0].xyz, VM[1].xyz, VM[2].xyz);

	vec3 vertexTangentCameraspace = mv3x3 * normalize(tangent);
	vec3 vertexBitangentCameraspace = mv3x3 * normalize(bitangent);
	vec3 vertexNormalCameraspace = mv3x3 * normalize(normal);

	mat3 TBN = transpose(mat3(
		vertexTangentCameraspace,
		vertexBitangentCameraspace,
		vertexNormalCameraspace	
	));

	gl_Position = PVM* vec4(position, 1);

	

	positionWorldSpace = (modelMatrix*vec4(position,1)).xyz;

	//Vetor ta space, the camera is at the origin (0,0,0)
	vec3 positionCameraSpace = (VM*vec4(position,1)).xzy;
	eyeDirectionCameraSpace = vec3(0,0,0) - positionCameraSpace;

	vec3 lightPositionCameraSpace = (viewMatrix * vec4(lightPositionWorldSpace,1)).xzy;
	lightDirectionCameraSpace = lightPositionCameraSpace + eyeDirectionCameraSpace;

	lightDirectionTangentspace = TBN * lightPositionCameraSpace;
	eyeDirectionTangentspace =  TBN * eyeDirectionCameraSpace;
}
