#include "ShadowMap.h"

using namespace cgue::thewall;

const GLfloat ShadowMap::g_quad_vertex_buffer_data[18] = { 
		-1.0f, -1.0f, 0.0f,
		 1.0f, -1.0f, 0.0f,
		-1.0f,  1.0f, 0.0f,
		-1.0f,  1.0f, 0.0f,
		 1.0f, -1.0f, 0.0f,
		 1.0f,  1.0f, 0.0f,
	};

ShadowMap::ShadowMap(int _WIDTH, int _HEIGHT) : WIDTH(_WIDTH),  HEIGHT(_HEIGHT)
{
	
	shaderDepth = new Shader(ORDNER + std::string("ShaderFile/vertexDP.vert"),
		ORDNER + std::string("ShaderFile/fragmentDP.frag"));
	shaderDepth->useShader();

	// Get a handle for our "MVP" uniform
	depthMatrixID = glGetUniformLocation(shaderDepth->programHandle, "depthMVP");

	// The framebuffer, which regroups 0, 1, or more textures, and 0 or 1 depth buffer.
	FramebufferName = 0;
	glGenFramebuffers(1, &FramebufferName);
	glBindFramebuffer(GL_FRAMEBUFFER, FramebufferName);

	
	glGenTextures(1, &depthTexture);
	glBindTexture(GL_TEXTURE_2D, depthTexture);
	glTexImage2D(GL_TEXTURE_2D, 0,GL_DEPTH_COMPONENT16, 1024, 1024, 0,GL_DEPTH_COMPONENT, GL_FLOAT, 0);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC, GL_LEQUAL);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_COMPARE_R_TO_TEXTURE);

	glFramebufferTexture(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, depthTexture, 0);

	// No color output in the bound framebuffer, only depth.
	glDrawBuffer(GL_NONE);

	
	glGenBuffers(1, &quad_vertexbuffer);
	glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
	glBufferData(GL_ARRAY_BUFFER, 3*6*sizeof(GLfloat), g_quad_vertex_buffer_data, GL_STATIC_DRAW);
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
	shaderQuad = new Shader(ORDNER + std::string("ShaderFile/vertexSP.vert"),
		ORDNER + std::string("ShaderFile/fragmentSP.frag"));
	shaderQuad->useShader();
	texID = glGetUniformLocation(shaderQuad->programHandle, "texture");
}


ShadowMap::~ShadowMap(void)
{
	delete shaderQuad;
	delete shaderDepth;

	glDeleteFramebuffers(1, &FramebufferName);
	glDeleteTextures(1, &depthTexture);
	glDeleteBuffers(1, &quad_vertexbuffer);
}


void ShadowMap::initDepth(Shader * shader, GLuint& DepthBiasID, GLuint& ShadowMapID, Shader * shaderNormal, GLuint& DepthBiasIDNM, GLuint& ShadowMapIDNM){
	shader->useShader();
	DepthBiasID = glGetUniformLocation(shader->programHandle, "depthBiasVP");
	ShadowMapID = glGetUniformLocation(shader->programHandle, "shadowMap");
	shaderNormal->useShader();
	DepthBiasIDNM = glGetUniformLocation(shaderNormal->programHandle, "depthBiasVP");
	ShadowMapIDNM = glGetUniformLocation(shaderNormal->programHandle, "shadowMap");
}

void ShadowMap::drawDepth(MausKeyboard* mk, Ground* ground, Raven* raven, Stone* stone, Key* key[], Tree* tree[], int treelength, GLuint DepthBiasID, GLuint DepthBiasIDNM, 
				glm::vec3 lightPos, glm::vec3 lightInvDir, Shader* shader, Shader* shaderNormal){

	glBindFramebuffer(GL_FRAMEBUFFER, FramebufferName);
	glViewport(0,0, 1024, 1024);

	
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);

	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	
	//Update
	shaderDepth->useShader();

	
	//TODO Schauen ob das mit den light dir und so passt!!!


	float xx = 80.0f;
	glm::mat4 depthProjectionMatrix = glm::ortho<float>(-xx,xx,-xx,xx,-xx,2*xx);
	glm::mat4 depthViewMatrix = glm::lookAt(lightPos, lightInvDir, glm::vec3(0,1,0));

	
	glm::mat4 depthModelMatrix = glm::mat4(1.0);
	glm::mat4 depthMVP = depthProjectionMatrix * depthViewMatrix * glm::mat4(1.0f);
	
	//DRAW

	for(int x = 0; x < treelength; x++){
		glm::mat4 mm = tree[x]->modelMatrix;
		mm[3][1] -= 3.0f;
		depthMVP = depthProjectionMatrix * depthViewMatrix * mm;
		glUniformMatrix4fv(depthMatrixID, 1, GL_FALSE, &depthMVP[0][0]);
		tree[x]->drawDepth();
	}
	depthMVP = depthProjectionMatrix * depthViewMatrix * ground->modelMatrix;
	glUniformMatrix4fv(depthMatrixID, 1, GL_FALSE, &depthMVP[0][0]);
	ground->drawDepth();
	depthMVP = depthProjectionMatrix * depthViewMatrix * ground->wall->modelMatrix;
	glUniformMatrix4fv(depthMatrixID, 1, GL_FALSE, &depthMVP[0][0]);
	ground->wall->drawDepth();

	glBindFramebuffer(GL_FRAMEBUFFER, 0);
	glViewport(0,0, 1024, 1024);

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glm::mat4 biasMatrix(
		0.5, 0.0, 0.0, 0.0, 
		0.0, 0.5, 0.0, 0.0,
		0.0, 0.0, 0.5, 0.0,
		0.5, 0.5, 0.5, 1.0
	);
	glm::mat4 depthBiasVP = biasMatrix*depthProjectionMatrix * depthViewMatrix;
	
	shader->useShader();

	glUniformMatrix4fv(DepthBiasID, 1, GL_FALSE, &depthBiasVP[0][0]);

	shaderNormal->useShader();

	glUniformMatrix4fv(DepthBiasIDNM, 1, GL_FALSE, &depthBiasVP[0][0]);
}

void ShadowMap::activateDepthTexture(GLuint ShadowMapID){
	glActiveTexture(GL_TEXTURE3);
	glBindTexture(GL_TEXTURE_2D, depthTexture);
	glUniform1i(ShadowMapID, 3);
}

	//shadowMap->initDepth(shader, DepthBiasID, ShadowMapID, shaderNormal, DepthBiasIDNM, ShadowMapIDNM);
	//shadowMap->drawDepth(mk, ground, raven, stone, key, tree, treelength, DepthBiasID, DepthBiasIDNM, 
	//			glm::vec3(0.0f,15.0f,0.0f), glm::vec3(0,0,0), shader, shaderNormal);
	//shader->useShader();
	//shadowMap->activateDepthTexture(ShadowMapID);
	//shaderNormal->useShader();
	//shadowMap->activateDepthTexture(ShadowMapIDNM);

void ShadowMap::drawAfter(){

	glViewport(0,0,1024/2, 1024/2);

	shaderQuad->useShader();

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, depthTexture);
	glUniform1i(texID, 0);

	glEnableVertexAttribArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, quad_vertexbuffer);
	glVertexAttribPointer(
		0,
		3,
		GL_FLOAT,
		GL_FALSE,
		0,
		(void*)0
	);

	glDrawArrays(GL_TRIANGLES, 0, 6);
	glDisableVertexAttribArray(0);
}