#include "Skybox.h"
#include "iostream"
using namespace cgue::thewall;

Skybox::Skybox(Shader* _shader) : shader(_shader)
{
	tTextures[SKYFRONT] = new Texture(ORDNER + std::string("Texturen/front.jpg"));
	tTextures[SKYBACK] = new Texture(ORDNER + std::string("Texturen/back.jpg"));
	tTextures[SKYLEFT] = new Texture(ORDNER + std::string("Texturen/left.jpg"));
	tTextures[SKYRIGHT] = new Texture(ORDNER + std::string("Texturen/right.jpg"));
	tTextures[SKYUP] = new Texture(ORDNER + std::string("Texturen/top.jpg"));
	tTextures[SKYDOWN] = new Texture(ORDNER + std::string("Texturen/down.jpg"));
	glm::vec3 vSkyBoxVertices[24] = 
	{
		// Front face
		glm::vec3(200.0f, 200.0f, 200.0f), glm::vec3(200.0f, -200.0f, 200.0f), glm::vec3(-200.0f, 200.0f, 200.0f), glm::vec3(-200.0f, -200.0f, 200.0f),
		// Back face
		glm::vec3(-200.0f, 200.0f, -200.0f), glm::vec3(-200.0f, -200.0f, -200.0f), glm::vec3(200.0f, 200.0f, -200.0f), glm::vec3(200.0f, -200.0f, -200.0f),
		// Left face
		glm::vec3(-200.0f, 200.0f, 200.0f), glm::vec3(-200.0f, -200.0f, 200.0f), glm::vec3(-200.0f, 200.0f, -200.0f), glm::vec3(-200.0f, -200.0f, -200.0f),
		// Right face
		glm::vec3(200.0f, 200.0f, -200.0f), glm::vec3(200.0f, -200.0f, -200.0f), glm::vec3(200.0f, 200.0f, 200.0f), glm::vec3(200.0f, -200.0f, 200.0f),
		// Top face
		glm::vec3(-200.0f, 200.0f, -200.0f), glm::vec3(200.0f, 200.0f, -200.0f), glm::vec3(-200.0f, 200.0f, 200.0f), glm::vec3(200.0f, 200.0f, 200.0f),
		// Bottom face
		glm::vec3(200.0f, -200.0f, -200.0f), glm::vec3(-200.0f, -200.0f, -200.0f), glm::vec3(200.0f, -200.0f, 200.0f), glm::vec3(-200.0f, -200.0f, 200.0f),
	};

	glm::vec2 vSkyBoxTexCoords[4] =
	{
		glm::vec2(0.0f, 1.0f), glm::vec2(0.0f, 0.0f), glm::vec2(1.0f, 1.0f), glm::vec2(1.0f, 0.0f)
	};
	glm::vec3 vSkyBoxNormals[6] = 
	{
		glm::vec3(0.0f, 0.0f, -1.0f),
		glm::vec3(0.0f, 0.0f, 1.0f),
		glm::vec3(1.0f, 0.0f, 0.0f),
		glm::vec3(-1.0f, 0.0f, 0.0f),
		glm::vec3(0.0f, -1.0f, 0.0f),
		glm::vec3(0.0f, 1.0f, 0.0f)
	};

	for(int i = 0; i <24; i++){
		vertices.push_back(vSkyBoxVertices[i]);
		uvs.push_back(vSkyBoxTexCoords[i%4]);
		normals.push_back(vSkyBoxNormals[i/4]);
	}

	glGenBuffers(1, &positionBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	glBufferData(GL_ARRAY_BUFFER, vertices.size() * sizeof(glm::vec3), &vertices[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glGenBuffers(1, &uvBuffer);
	glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
	glBufferData(GL_ARRAY_BUFFER, uvs.size() * sizeof(glm::vec2), &uvs[0], GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// Generate VAO
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	//Buffer binden
	glBindBuffer(GL_ARRAY_BUFFER,positionBuffer);
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindBuffer(GL_ARRAY_BUFFER,uvBuffer);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, 0);

	glBindVertexArray(0);


	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
	glBindBuffer(GL_ARRAY_BUFFER,0);
}

void Skybox::draw()
{
	glDepthMask(0);
	glBindVertexArray(vao);

	for(int i= 0; i < 6; i++)
	{
		tTextures[i]->Bind(0);
		glUniform1i(glGetUniformLocation(shader->programHandle, "colorTexture"), 0);
		glDrawArrays(GL_TRIANGLES, i*4, 4);
	}
	
	glBindVertexArray(0);
	
	glDepthMask(1);
}

Skybox::~Skybox()
{
	for(int i= 0; i < 6; i++)
		delete(tTextures[i]);
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteBuffers(1, &vao);
}